--
library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity SDREADIF is
	port (
		SD_DAT0		: in  std_logic;						-- from SD card
		SD_WP		: in  std_logic;						-- from SD card
		ADD			: in  std_logic_vector(9 downto 0);		-- from CPU
		DATAI		: in  std_logic_vector(7 downto 0);		-- from CPU
		WRN			: in  std_logic;						-- from CPU
		RDN			: in  std_logic;						-- from CPU
		CMTRDOPEN	: in  std_logic;						-- from 8049
		CMTRDREQ	: in  std_logic;						-- from 8049
		CMTRDACC	: in  std_logic;						-- from 8049
		CMTRDACCMD	: in  std_logic;
		CMTWROPEN	: in  std_logic;						-- from 8049
		CMTWRREQ	: in  std_logic;						-- from 8049
		CMTWRDT		: in  std_logic_vector(7 downto 0);		-- from 8049
		DETBLK		: in  std_logic;
		CLK50M		: in  std_logic;
		RSTN		: in  std_logic;
		DATAO		: out std_logic_vector(7 downto 0);		-- to CPU
		SD_RDAD		: out std_logic_vector(18 downto 0);	-- to SDRAM/CGROM
		SD_RDDT		: out std_logic_vector(7 downto 0);		-- to SDRAM/CGROM
		SD_SDENB	: out std_logic;						-- to SDRAM
		SD_SDWRN	: out std_logic;						-- to SDRAM
		SD_RDDONE	: out std_logic;
		SD_ERR		: out std_logic;
		SD_OUTENB	: out std_logic;
		CMTRDRDY	: out std_logic;						-- to 8049
		CMTWRRDY	: out std_logic;						-- to 8049
		CMTRDDT		: out std_logic_vector(7 downto 0);		-- to 8049
		CMTCNT		: out std_logic_vector(15 downto 0);
		ACCCNT		: out std_logic_vector(31 downto 0);
		SD_CMD		: out std_logic;						-- to SD card
		SD_DAT		: out std_logic_vector(3 downto 1);		-- to SD card
		SD_CLK		: out std_logic;						-- to SD card
		MONOUT		: out std_logic_vector(28 downto 0)
	);
end SDREADIF;

architecture RTL of SDREADIF is

	component SDREAD_CMDCNT is
		port (
			SD_DAT0		: in  std_logic;
			SD_WP		: in  std_logic;
			CMD_START	: in  std_logic;
			CMD_NO		: in  std_logic_vector(5 downto 0);
			SDADD		: in  std_logic_vector(31 downto 0);
			SDCNTENB	: in  std_logic;
			OUTDT		: in  std_logic_vector(7 downto 0);
			CLK			: in  std_logic;
			RSTN		: in  std_logic;
			STATEOUT	: out std_logic_vector(7 downto 0);
			INDT		: out std_logic_vector(7 downto 0);
			INDTENB		: out std_logic;
			OUTDTENB	: out std_logic;
			INDTLT		: out std_logic;
			INDTLT2		: out std_logic;
			CMD_END		: out std_logic;
			IDLEDET		: out std_logic;
			ERRDET		: out std_logic;
			SD_CMD		: out std_logic;
			SD_DAT		: out std_logic_vector(3 downto 1);
			SD_CLK		: out std_logic
		);
	end component;

	component SPRAM_2048W8B is
		port (
			address	: in  std_logic_vector(10 downto 0);
			data	: in  std_logic_vector(7 downto 0);
			wren	: in  std_logic;
			clock	: in  std_logic;
			aclr	: in  std_logic;
			q		: out std_logic_vector(7 downto 0)
		);
	end component;

	signal indt_i		: std_logic_vector(7 downto 0);
	signal indtenb_i	: std_logic;
	signal outdtenb_i	: std_logic;
	signal indtlt2_i	: std_logic;
	signal cmd_end_i	: std_logic;
	signal idledet_i	: std_logic;
	signal errdet_i		: std_logic;
	signal cmd_end_f1	: std_logic;
	signal cmd_end_f2	: std_logic;
	signal sdrdenb		: std_logic;
	signal sdwrenb		: std_logic;

	signal sd_rdcnt		: std_logic_vector(8 downto 0);
	signal sd_wrcnt		: std_logic_vector(8 downto 0);

	signal fifo_sdramtmg	: std_logic_vector(4 downto 0);
	signal fifo_cmtrdadd	: std_logic_vector(31 downto 0);
	signal fifo_cmtwradd	: std_logic_vector(31 downto 0);
	signal fifo_fddrdadd	: std_logic_vector(31 downto 0);
	signal fifo_fddwradd	: std_logic_vector(31 downto 0);

	signal cmtrdrq_f1	: std_logic;
	signal cmtrdrq_f2	: std_logic;
	signal cmtrdrq_f3	: std_logic;
	signal cmtrdycnt	: std_logic_vector(23 downto 0);
	signal cmtrdrdy_i	: std_logic;
	signal cmtrddt_i	: std_logic_vector(7 downto 0);

	signal cmtwrrdy_i	: std_logic;

	signal aclr		: std_logic;
	signal buf_add	: std_logic_vector(10 downto 0);
	signal buf_dat	: std_logic_vector(7 downto 0);
	signal buf_wr	: std_logic;
	signal buf_q	: std_logic_vector(7 downto 0);

	signal reg_start		: std_logic;						-- 0x8200(0)
	signal reg_startend		: std_logic;						-- 0x8200(0)
	signal reg_idledet		: std_logic;						-- 0x8200(1)
	signal reg_errdet		: std_logic;						-- 0x8200(2)
	signal reg_inuse		: std_logic;						-- 0x8201(0)
	signal reg_sderr		: std_logic;						-- 0x8201(1)
	signal reg_sdrddone		: std_logic;						-- 0x8202(0)
	signal reg_rominit		: std_logic;						-- 0x8203(0)
	signal reg_rominitend	: std_logic;						-- 0x8203(0)
	signal reg_cmdno		: std_logic_vector(5 downto 0);		-- 0x8205(5:0)
	signal reg_sdadd		: std_logic_vector(31 downto 0);	-- 0x8206-0x8209
	signal reg_bufad		: std_logic_vector(1 downto 0);		-- 0x8210(1:0)
	signal reg_bufsel		: std_logic_vector(3 downto 0);		-- 0x8211(3:0)
	signal reg_sdrdinit		: std_logic;						-- 0x8213(0)
	signal reg_sdrdfull		: std_logic;						-- 0x8213(4)
	signal reg_sdwrinit		: std_logic;						-- 0x8214(0)
	signal reg_sdwrfull		: std_logic;						-- 0x8214(4)

	signal reg_cmtlen_0		: std_logic_vector(31 downto 0);	-- 0x8220-0x8223
	signal reg_cmtptr_0		: std_logic_vector(31 downto 0);	-- 0x8224-0x8227
	signal reg_cl_0			: std_logic_vector(15 downto 0);	-- 0x8228-0x8229
	signal reg_sec_0		: std_logic_vector( 7 downto 0);	-- 0x822A
	signal reg_stcl_0		: std_logic_vector(15 downto 0);	-- 0x822B-0x822C
	signal reg_befcl_0		: std_logic_vector(15 downto 0);	-- 0x822D-0x822E
	signal reg_dircl_0		: std_logic_vector(15 downto 0);	-- 0x822F-0x8230
	signal reg_dirpos_0		: std_logic_vector(15 downto 0);	-- 0x8231-0x8232
	signal reg_fatnew_0		: std_logic_vector( 7 downto 0);	-- 0x8233
	signal reg_cmtrdst		: std_logic;						-- 0x8234(0)
	signal reg_cmtrdinit	: std_logic;						-- 0x8234(1)
	signal reg_cmtrdld		: std_logic;						-- 0x8234(2)
	signal reg_cmtrdfull	: std_logic;						-- 0x8234(4)
	signal reg_cmtrdend		: std_logic;						-- 0x8234(5)

	signal reg_cmtlen_1		: std_logic_vector(31 downto 0);	-- 0x8240-0x8243
	signal reg_cmtptr_1		: std_logic_vector(31 downto 0);	-- 0x8244-0x8247
	signal reg_cl_1			: std_logic_vector(15 downto 0);	-- 0x8248-0x8249
	signal reg_sec_1		: std_logic_vector( 7 downto 0);	-- 0x824A
	signal reg_stcl_1		: std_logic_vector(15 downto 0);	-- 0x824B-0x824C
	signal reg_befcl_1		: std_logic_vector(15 downto 0);	-- 0x824D-0x824E
	signal reg_dircl_1		: std_logic_vector(15 downto 0);	-- 0x824F-0x8250
	signal reg_dirpos_1		: std_logic_vector(15 downto 0);	-- 0x8251-0x8252
	signal reg_fatnew_1		: std_logic_vector( 7 downto 0);	-- 0x8253
	signal reg_cmtwrst		: std_logic;						-- 0x8254(0)
	signal reg_cmtwrld		: std_logic;						-- 0x8254(1)
	signal reg_cmtwrok		: std_logic;						-- 0x8254(2)
	signal reg_cmtwrfull	: std_logic;						-- 0x8254(4)

	signal reg_cmtlen_2		: std_logic_vector(31 downto 0);	-- 0x8260-0x8263
	signal reg_cmtptr_2		: std_logic_vector(31 downto 0);	-- 0x8264-0x8267
	signal reg_cl_2			: std_logic_vector(15 downto 0);	-- 0x8268-0x8269
	signal reg_sec_2		: std_logic_vector( 7 downto 0);	-- 0x826A
	signal reg_stcl_2		: std_logic_vector(15 downto 0);	-- 0x826B-0x826C
	signal reg_befcl_2		: std_logic_vector(15 downto 0);	-- 0x826D-0x826E
	signal reg_dircl_2		: std_logic_vector(15 downto 0);	-- 0x826F-0x8270
	signal reg_dirpos_2		: std_logic_vector(15 downto 0);	-- 0x8271-0x8272
	signal reg_fatnew_2		: std_logic_vector( 7 downto 0);	-- 0x8273

	signal reg_cmtwnom		: std_logic_vector(31 downto 0);	-- 0x8300-0x8303
	signal reg_cmtwacc		: std_logic_vector(31 downto 0);	-- 0x8304-0x8307
	signal reg_cmtwst		: std_logic_vector(31 downto 0);	-- 0x8308-0x830B
	signal reg_acccnt		: std_logic_vector(31 downto 0);	-- 0x830C-0x830F

	signal reg_start_f1	: std_logic;
	signal reg_start_f2	: std_logic;
	signal reg_start_f3	: std_logic;
	signal reg_start_r	: std_logic;

	signal reg_startlt1	: std_logic;
	signal reg_startlt2	: std_logic;

	signal reg_rominit_f1	: std_logic;
	signal reg_rominit_f2	: std_logic;
	signal reg_rominit_f3	: std_logic;
	signal reg_rominit_r	: std_logic;
	signal rominitcnt		: std_logic_vector(17 downto 0);

	signal cmtrdopen_f1	: std_logic;
	signal cmtrdopen_f2	: std_logic;
	signal cmtrdreq_f1	: std_logic;
	signal cmtrdreq_f2	: std_logic;
	signal cmtwropen_f1	: std_logic;
	signal cmtwropen_f2	: std_logic;
	signal cmtwrreq_f1	: std_logic;
	signal cmtwrreq_f2	: std_logic;
	signal cmtwrreq_f3	: std_logic;
	signal cmtwrreq_r	: std_logic;
	signal detblk_f1	: std_logic;
	signal detblk_f2	: std_logic;

	signal cmtwr_lt	: std_logic;

	signal cmtwrdt_f1	: std_logic_vector(7 downto 0);
	signal cmtwrdt_f2	: std_logic_vector(7 downto 0);
	signal cmtwrdt_f3	: std_logic_vector(7 downto 0);

	signal reg_cmtwrst_f1	: std_logic;
	signal reg_cmtwrst_f2	: std_logic;
	signal reg_cmtwrst_f3	: std_logic;

	signal fddwrdt_f3	: std_logic_vector(7 downto 0);
	signal fddwrreq_r	: std_logic;

	signal datao_i		: std_logic_vector(7 downto 0);
	signal sd_rdad_i	: std_logic_vector(18 downto 0);
	signal sd_rddt_i	: std_logic_vector(7 downto 0);
	signal sd_sdenb_i	: std_logic;
	signal sd_sdwrn_i	: std_logic;
	signal sd_cgenb_i	: std_logic;
	signal sd_cgwrn_i	: std_logic;
	signal sdadd_i		: std_logic_vector(31 downto 0);

	signal rdn_f1	: std_logic;
	signal rdn_f2	: std_logic;
	signal rdn_f3	: std_logic;
	signal wrn_f1	: std_logic;
	signal wrn_f2	: std_logic;
	signal wrn_f3	: std_logic;
	signal add_f1	: std_logic_vector(9 downto 0);
	signal add_f2	: std_logic_vector(9 downto 0);
	signal datai_f1	: std_logic_vector(7 downto 0);
	signal datai_f2	: std_logic_vector(7 downto 0);
	signal rdp		: std_logic;
	signal wrp		: std_logic;
	signal add_exp	: std_logic_vector(15 downto 0);

begin

	U_SDREAD_CMDCNT : SDREAD_CMDCNT
	port map (
		SD_DAT0		=> SD_DAT0,
		SD_WP		=> SD_WP,
		CMD_START	=> reg_startlt2,
		CMD_NO		=> reg_cmdno,
		SDADD		=> sdadd_i,
		SDCNTENB	=> reg_inuse,
		OUTDT		=> buf_q,
		CLK			=> CLK50M,
		RSTN		=> RSTN,
		STATEOUT	=> open,
		INDT		=> indt_i,
		INDTENB		=> indtenb_i,
		OUTDTENB	=> outdtenb_i,
		INDTLT2		=> indtlt2_i,
		CMD_END		=> cmd_end_i,
		IDLEDET		=> idledet_i,
		ERRDET		=> errdet_i,
		SD_CMD		=> SD_CMD,
		SD_DAT		=> SD_DAT,
		SD_CLK		=> SD_CLK
	);

	sdadd_i <= reg_sdadd(22 downto 0) & "000000000";

	U_SPRAM_2048W8B : SPRAM_2048W8B
	port map (
		address	=> buf_add,
		data	=> buf_dat,
		wren	=> buf_wr,
		clock	=> CLK50M,
		aclr	=> aclr,
		q		=> buf_q
	);

	aclr     <= not RSTN;

	buf_add <=	reg_bufad & sd_wrcnt                  when (reg_bufsel = "0000") else
				reg_bufad & add_f2(8 downto 0)        when (reg_bufsel = "0001") else
				reg_bufad & fifo_cmtrdadd(8 downto 0) when (reg_bufsel = "0100") else
				reg_bufad & fifo_cmtrdadd(8 downto 0) when (reg_bufsel = "0101") else
				reg_bufad & fifo_fddrdadd(8 downto 0) when (reg_bufsel = "0110") else
				reg_bufad & sd_rdcnt                  when (reg_bufsel = "1000") else
				reg_bufad & add_f2(8 downto 0)        when (reg_bufsel = "1001") else
				reg_bufad & fifo_cmtwradd(8 downto 0) when (reg_bufsel = "1100") else
				reg_bufad & fifo_fddwradd(8 downto 0) when (reg_bufsel = "1110") else
				(others => '0');

	buf_dat <=	indt_i     when (reg_bufsel = "1000") else
				datai_f2   when (reg_bufsel = "1001") else
				cmtwrdt_f3 when (reg_bufsel = "1100") else
				fddwrdt_f3 when (reg_bufsel = "1110") else
				X"00";

	buf_wr <=	sdrdenb    when (reg_bufsel = "1000") else
				wrp        when (reg_bufsel = "1001") else
				cmtwrreq_r when (reg_bufsel = "1100") else
				fddwrreq_r when (reg_bufsel = "1110") else
				'0';

	MONOUT(28)           <= '0';

	MONOUT(27)           <= reg_cmtwrok;
	MONOUT(26)           <= reg_cmtwrld;
	MONOUT(25)           <= reg_cmtwrst;
	MONOUT(24)           <= cmtwrreq_r;
	MONOUT(23 downto 16) <= cmtwrdt_f3;
	MONOUT(15)           <= reg_cmtwrfull;
	MONOUT(14)           <= sdrdenb;
	MONOUT(13)           <= wrp;
	MONOUT(12 downto  9) <= reg_bufsel;
	MONOUT( 8 downto  0) <= fifo_cmtwradd(8 downto 0);

	fddwrdt_f3    <= X"00";
	fddwrreq_r    <= '0';
	fifo_fddrdadd <= (others => '0');
	fifo_fddwradd <= (others => '0');


-- CPU I/F
-- data latch
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			rdn_f1   <= '1';
			rdn_f2   <= '1';
			rdn_f3   <= '1';
			wrn_f1   <= '1';
			wrn_f2   <= '1';
			wrn_f3   <= '1';
			add_f1   <= (others => '0');
			add_f2   <= (others => '0');
			datai_f1 <= (others => '0');
			datai_f2 <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			rdn_f1   <= RDN;
			rdn_f2   <= rdn_f1;
			rdn_f3   <= rdn_f2;
			wrn_f1   <= WRN;
			wrn_f2   <= wrn_f1;
			wrn_f3   <= wrn_f2;
			add_f1   <= ADD;
			add_f2   <= add_f1;
			datai_f1 <= DATAI;
			datai_f2 <= datai_f1;
		end if;
	end process;

	rdp <= '1' when (rdn_f2 = '0' and rdn_f3 = '1') else '0';
	wrp <= '1' when (wrn_f2 = '0' and wrn_f3 = '1') else '0';

	add_exp <= "100000" & add_f2;

-- register
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			reg_start    <= '0';
			reg_inuse    <= '0';
			reg_sderr    <= '0';
			reg_sdrddone <= '0';
			reg_rominit  <= '0';
			reg_cmdno    <= (others => '0');
			reg_sdadd    <= (others => '0');
			reg_bufad    <= (others => '0');
			reg_bufsel   <= (others => '0');
			reg_sdrdinit <= '0';
			reg_sdwrinit <= '0';
			reg_cmtlen_0 <= (others => '0');
			reg_cmtptr_0 <= (others => '0');
			reg_cl_0     <= (others => '0');
			reg_sec_0    <= (others => '0');
			reg_stcl_0   <= (others => '0');
			reg_befcl_0  <= (others => '0');
			reg_dircl_0  <= (others => '0');
			reg_dirpos_0 <= (others => '0');
			reg_fatnew_0 <= (others => '0');
			reg_cmtrdst  <= '0';
			reg_cmtrdinit<= '0';
			reg_cmtrdld  <= '0';
			reg_cmtlen_1 <= (others => '0');
			reg_cmtptr_1 <= (others => '0');
			reg_cl_1     <= (others => '0');
			reg_sec_1    <= (others => '0');
			reg_stcl_1   <= (others => '0');
			reg_befcl_1  <= (others => '0');
			reg_dircl_1  <= (others => '0');
			reg_dirpos_1 <= (others => '0');
			reg_fatnew_1 <= (others => '0');
			reg_cmtwrst  <= '0';
			reg_cmtwrld  <= '0';
			reg_cmtwrok  <= '0';
			reg_cmtlen_2 <= (others => '0');
			reg_cmtptr_2 <= (others => '0');
			reg_cl_2     <= (others => '0');
			reg_sec_2    <= (others => '0');
			reg_stcl_2   <= (others => '0');
			reg_befcl_2  <= (others => '0');
			reg_dircl_2  <= (others => '0');
			reg_dirpos_2 <= (others => '0');
			reg_fatnew_2 <= (others => '0');
			reg_cmtwnom  <= X"0007EEEE";
			reg_cmtwacc  <= X"00011000";
			reg_cmtwst   <= X"00120000";
			reg_acccnt   <= X"03000000";
		elsif (CLK50M'event and CLK50M = '1') then
			if (wrp = '1') then
				case add_exp is
					when X"8200" => reg_start                  <= datai_f2(0);
					when X"8201" =>	reg_inuse                  <= datai_f2(0);
									reg_sderr                  <= datai_f2(1);
					when X"8202" => reg_sdrddone               <= datai_f2(0);
					when X"8203" => reg_rominit                <= datai_f2(0);
					when X"8205" => reg_cmdno                  <= datai_f2(5 downto 0);
					when X"8206" => reg_sdadd( 7 downto  0)    <= datai_f2;
					when X"8207" => reg_sdadd(15 downto  8)    <= datai_f2;
					when X"8208" => reg_sdadd(23 downto 16)    <= datai_f2;
					when X"8209" => reg_sdadd(31 downto 24)    <= datai_f2;
					when X"8210" => reg_bufad                  <= datai_f2(1 downto 0);
					when X"8211" => reg_bufsel                 <= datai_f2(3 downto 0);
					when X"8213" => reg_sdrdinit               <= datai_f2(0);
					when X"8214" => reg_sdwrinit               <= datai_f2(0);
					when X"8220" => reg_cmtlen_0( 7 downto  0) <= datai_f2;
					when X"8221" => reg_cmtlen_0(15 downto  8) <= datai_f2;
					when X"8222" => reg_cmtlen_0(23 downto 16) <= datai_f2;
					when X"8223" => reg_cmtlen_0(31 downto 24) <= datai_f2;
					when X"8224" => reg_cmtptr_0( 7 downto  0) <= datai_f2;
					when X"8225" => reg_cmtptr_0(15 downto  8) <= datai_f2;
					when X"8226" => reg_cmtptr_0(23 downto 16) <= datai_f2;
					when X"8227" => reg_cmtptr_0(31 downto 24) <= datai_f2;
					when X"8228" => reg_cl_0( 7 downto  0)     <= datai_f2;
					when X"8229" => reg_cl_0(15 downto  8)     <= datai_f2;
					when X"822A" => reg_sec_0                  <= datai_f2;
					when X"822B" => reg_stcl_0( 7 downto  0)   <= datai_f2;
					when X"822C" => reg_stcl_0(15 downto  8)   <= datai_f2;
					when X"822D" => reg_befcl_0( 7 downto  0)  <= datai_f2;
					when X"822E" => reg_befcl_0(15 downto  8)  <= datai_f2;
					when X"822F" => reg_dircl_0( 7 downto  0)  <= datai_f2;
					when X"8230" => reg_dircl_0(15 downto  8)  <= datai_f2;
					when X"8231" => reg_dirpos_0( 7 downto  0) <= datai_f2;
					when X"8232" => reg_dirpos_0(15 downto  8) <= datai_f2;
					when X"8233" => reg_fatnew_0               <= datai_f2;
					when X"8234" => reg_cmtrdst                <= datai_f2(0);
									reg_cmtrdinit              <= datai_f2(1);
									reg_cmtrdld                <= datai_f2(2);
					when X"8240" => reg_cmtlen_1( 7 downto  0) <= datai_f2;
					when X"8241" => reg_cmtlen_1(15 downto  8) <= datai_f2;
					when X"8242" => reg_cmtlen_1(23 downto 16) <= datai_f2;
					when X"8243" => reg_cmtlen_1(31 downto 24) <= datai_f2;
					when X"8244" => reg_cmtptr_1( 7 downto  0) <= datai_f2;
					when X"8245" => reg_cmtptr_1(15 downto  8) <= datai_f2;
					when X"8246" => reg_cmtptr_1(23 downto 16) <= datai_f2;
					when X"8247" => reg_cmtptr_1(31 downto 24) <= datai_f2;
					when X"8248" => reg_cl_1( 7 downto  0)     <= datai_f2;
					when X"8249" => reg_cl_1(15 downto  8)     <= datai_f2;
					when X"824A" => reg_sec_1                  <= datai_f2;
					when X"824B" => reg_stcl_1( 7 downto  0)   <= datai_f2;
					when X"824C" => reg_stcl_1(15 downto  8)   <= datai_f2;
					when X"824D" => reg_befcl_1( 7 downto  0)  <= datai_f2;
					when X"824E" => reg_befcl_1(15 downto  8)  <= datai_f2;
					when X"824F" => reg_dircl_1( 7 downto  0)  <= datai_f2;
					when X"8250" => reg_dircl_1(15 downto  8)  <= datai_f2;
					when X"8251" => reg_dirpos_1( 7 downto 0)  <= datai_f2;
					when X"8252" => reg_dirpos_1(15 downto 8)  <= datai_f2;
					when X"8253" => reg_fatnew_1               <= datai_f2;
					when X"8254" => reg_cmtwrst                <= datai_f2(0);
									reg_cmtwrld                <= datai_f2(1);
									reg_cmtwrok                <= datai_f2(2);
					when X"8260" => reg_cmtlen_2( 7 downto  0) <= datai_f2;
					when X"8261" => reg_cmtlen_2(15 downto  8) <= datai_f2;
					when X"8262" => reg_cmtlen_2(23 downto 16) <= datai_f2;
					when X"8263" => reg_cmtlen_2(31 downto 24) <= datai_f2;
					when X"8264" => reg_cmtptr_2( 7 downto  0) <= datai_f2;
					when X"8265" => reg_cmtptr_2(15 downto  8) <= datai_f2;
					when X"8266" => reg_cmtptr_2(23 downto 16) <= datai_f2;
					when X"8267" => reg_cmtptr_2(31 downto 24) <= datai_f2;
					when X"8268" => reg_cl_2( 7 downto  0)     <= datai_f2;
					when X"8269" => reg_cl_2(15 downto  8)     <= datai_f2;
					when X"826A" => reg_sec_2                  <= datai_f2;
					when X"826B" => reg_stcl_2( 7 downto  0)   <= datai_f2;
					when X"826C" => reg_stcl_2(15 downto  8)   <= datai_f2;
					when X"826D" => reg_befcl_2( 7 downto  0)  <= datai_f2;
					when X"826E" => reg_befcl_2(15 downto  8)  <= datai_f2;
					when X"826F" => reg_dircl_2( 7 downto  0)  <= datai_f2;
					when X"8270" => reg_dircl_2(15 downto  8)  <= datai_f2;
					when X"8271" => reg_dirpos_2( 7 downto  0) <= datai_f2;
					when X"8272" => reg_dirpos_2(15 downto  8) <= datai_f2;
					when X"8273" => reg_fatnew_2               <= datai_f2;
					when X"8300" => reg_cmtwnom( 7 downto  0)  <= datai_f2;
					when X"8301" => reg_cmtwnom(15 downto  8)  <= datai_f2;
					when X"8302" => reg_cmtwnom(23 downto 16)  <= datai_f2;
					when X"8303" => reg_cmtwnom(31 downto 24)  <= datai_f2;
					when X"8304" => reg_cmtwacc( 7 downto  0)  <= datai_f2;
					when X"8305" => reg_cmtwacc(15 downto  8)  <= datai_f2;
					when X"8306" => reg_cmtwacc(23 downto 16)  <= datai_f2;
					when X"8307" => reg_cmtwacc(31 downto 24)  <= datai_f2;
					when X"8308" => reg_cmtwst( 7 downto  0)   <= datai_f2;
					when X"8309" => reg_cmtwst(15 downto  8)   <= datai_f2;
					when X"830A" => reg_cmtwst(23 downto 16)   <= datai_f2;
					when X"830B" => reg_cmtwst(31 downto 24)   <= datai_f2;
					when X"830C" => reg_acccnt( 7 downto  0)   <= datai_f2;
					when X"830D" => reg_acccnt(15 downto  8)   <= datai_f2;
					when X"830E" => reg_acccnt(23 downto 16)   <= datai_f2;
					when X"830F" => reg_acccnt(31 downto 24)   <= datai_f2;
					when others      => null;
				end case;
			end if;
		end if;
	end process;

-- start_end register
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			cmd_end_f1 <= '0';
			cmd_end_f2 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			cmd_end_f1 <= cmd_end_i;
			cmd_end_f2 <= cmd_end_f1;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			reg_start_f1 <= '0';
			reg_start_f2 <= '0';
			reg_start_f3 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			reg_start_f1 <= reg_start;
			reg_start_f2 <= reg_start_f1;
			reg_start_f3 <= reg_start_f2;
		end if;
	end process;

	reg_start_r <= '1' when (reg_start_f2 = '1' and reg_start_f3 = '0') else '0';

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			reg_startlt1 <= '0';
			reg_startlt2 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (reg_start_r = '1') then
				reg_startlt1 <= '1';
			elsif (indtlt2_i = '1') then
				reg_startlt1 <= '0';
			end if;

			if (indtlt2_i = '1') then
				reg_startlt2 <= reg_startlt1;
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			reg_startend <= '0';
			reg_idledet  <= '0';
			reg_errdet   <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (cmd_end_f1 = '1' and cmd_end_f2 = '0') then
				reg_startend <= '0';
			elsif (reg_start_r = '1') then
				reg_startend <= '1';
			end if;
			if (cmd_end_f1 = '1' and cmd_end_f2 = '0') then
				reg_idledet  <= idledet_i;
				reg_errdet   <= errdet_i;
			end if;
		end if;
	end process;

-- register read
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			datao_i <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if (rdp = '1') then
				case add_exp is
					when X"8200" => datao_i <= "00000" & reg_errdet & reg_idledet & reg_startend;
					when X"8201" =>	datao_i <= "000000" & reg_sderr & reg_inuse;
					when X"8202" =>	datao_i <= "0000000" & reg_sdrddone;
					when X"8203" =>	datao_i <= "0000000" & reg_rominitend;
					when X"8204" => datao_i <= "0000000" & SD_WP;
					when X"8205" => datao_i <= "00" & reg_cmdno;
					when X"8206" => datao_i <= reg_sdadd( 7 downto  0);
					when X"8207" => datao_i <= reg_sdadd(15 downto  8);
					when X"8208" => datao_i <= reg_sdadd(23 downto 16);
					when X"8209" => datao_i <= reg_sdadd(31 downto 24);
					when X"8210" => datao_i <= "000000" & reg_bufad;
					when X"8211" => datao_i <= "0000" & reg_bufsel;
					when X"8212" => datao_i <= "000000" & cmtwropen_f2 & cmtrdopen_f2;
					when X"8213" => datao_i <= "000" & reg_sdrdfull & "000" & reg_sdrdinit;
					when X"8214" => datao_i <= "000" & reg_sdwrfull & "000" & reg_sdwrinit;
					when X"8220" => datao_i <= reg_cmtlen_0( 7 downto  0);
					when X"8221" => datao_i <= reg_cmtlen_0(15 downto  8);
					when X"8222" => datao_i <= reg_cmtlen_0(23 downto 16);
					when X"8223" => datao_i <= reg_cmtlen_0(31 downto 24);
					when X"8224" => datao_i <= reg_cmtptr_0( 7 downto  0);
					when X"8225" => datao_i <= reg_cmtptr_0(15 downto  8);
					when X"8226" => datao_i <= reg_cmtptr_0(23 downto 16);
					when X"8227" => datao_i <= reg_cmtptr_0(31 downto 24);
					when X"8228" => datao_i <= reg_cl_0( 7 downto  0);
					when X"8229" => datao_i <= reg_cl_0(15 downto  8);
					when X"822A" => datao_i <= reg_sec_0;
					when X"822B" => datao_i <= reg_stcl_0( 7 downto  0);
					when X"822C" => datao_i <= reg_stcl_0(15 downto  8);
					when X"822D" => datao_i <= reg_befcl_0( 7 downto  0);
					when X"822E" => datao_i <= reg_befcl_0(15 downto  8);
					when X"822F" => datao_i <= reg_dircl_0( 7 downto  0);
					when X"8230" => datao_i <= reg_dircl_0(15 downto  8);
					when X"8231" => datao_i <= reg_dirpos_0( 7 downto  0);
					when X"8232" => datao_i <= reg_dirpos_0(15 downto  8);
					when X"8233" => datao_i <= reg_fatnew_0;
					when X"8234" => datao_i <=	"00" & reg_cmtrdend & reg_cmtrdfull &
												"0" & reg_cmtrdld & reg_cmtrdinit & reg_cmtrdst;
					when X"8240" => datao_i <= reg_cmtlen_1( 7 downto  0);
					when X"8241" => datao_i <= reg_cmtlen_1(15 downto  8);
					when X"8242" => datao_i <= reg_cmtlen_1(23 downto 16);
					when X"8243" => datao_i <= reg_cmtlen_1(31 downto 24);
					when X"8244" => datao_i <= reg_cmtptr_1( 7 downto  0);
					when X"8245" => datao_i <= reg_cmtptr_1(15 downto  8);
					when X"8246" => datao_i <= reg_cmtptr_1(23 downto 16);
					when X"8247" => datao_i <= reg_cmtptr_1(31 downto 24);
					when X"8248" => datao_i <= reg_cl_1( 7 downto  0);
					when X"8249" => datao_i <= reg_cl_1(15 downto  8);
					when X"824A" => datao_i <= reg_sec_1;
					when X"824B" => datao_i <= reg_stcl_1( 7 downto  0);
					when X"824C" => datao_i <= reg_stcl_1(15 downto  8);
					when X"824D" => datao_i <= reg_befcl_1( 7 downto  0);
					when X"824E" => datao_i <= reg_befcl_1(15 downto  8);
					when X"824F" => datao_i <= reg_dircl_1( 7 downto  0);
					when X"8250" => datao_i <= reg_dircl_1(15 downto  8);
					when X"8251" => datao_i <= reg_dirpos_1( 7 downto  0);
					when X"8252" => datao_i <= reg_dirpos_1(15 downto  8);
					when X"8253" => datao_i <= reg_fatnew_1;
					when X"8254" => datao_i <=	"000" & reg_cmtwrfull &
												"0" & reg_cmtwrok & reg_cmtwrld & reg_cmtwrst;
					when X"8255" => datao_i <= fifo_cmtwradd( 7 downto  0);
					when X"8256" => datao_i <= fifo_cmtwradd(15 downto  8);
					when X"8257" => datao_i <= fifo_cmtwradd(23 downto 16);
					when X"8258" => datao_i <= fifo_cmtwradd(31 downto 24);
					when X"8260" => datao_i <= reg_cmtlen_2( 7 downto  0);
					when X"8261" => datao_i <= reg_cmtlen_2(15 downto  8);
					when X"8262" => datao_i <= reg_cmtlen_2(23 downto 16);
					when X"8263" => datao_i <= reg_cmtlen_2(31 downto 24);
					when X"8264" => datao_i <= reg_cmtptr_2( 7 downto  0);
					when X"8265" => datao_i <= reg_cmtptr_2(15 downto  8);
					when X"8266" => datao_i <= reg_cmtptr_2(23 downto 16);
					when X"8267" => datao_i <= reg_cmtptr_2(31 downto 24);
					when X"8268" => datao_i <= reg_cl_2( 7 downto  0);
					when X"8269" => datao_i <= reg_cl_2(15 downto  8);
					when X"826A" => datao_i <= reg_sec_2;
					when X"826B" => datao_i <= reg_stcl_2( 7 downto  0);
					when X"826C" => datao_i <= reg_stcl_2(15 downto  8);
					when X"826D" => datao_i <= reg_befcl_2( 7 downto  0);
					when X"826E" => datao_i <= reg_befcl_2(15 downto  8);
					when X"826F" => datao_i <= reg_dircl_2( 7 downto  0);
					when X"8270" => datao_i <= reg_dircl_2(15 downto  8);
					when X"8271" => datao_i <= reg_dirpos_2( 7 downto  0);
					when X"8272" => datao_i <= reg_dirpos_2(15 downto  8);
					when X"8273" => datao_i <= reg_fatnew_2;
					when X"8300" => datao_i <= reg_cmtwnom( 7 downto  0);
					when X"8301" => datao_i <= reg_cmtwnom(15 downto  8);
					when X"8302" => datao_i <= reg_cmtwnom(23 downto 16);
					when X"8303" => datao_i <= reg_cmtwnom(31 downto 24);
					when X"8304" => datao_i <= reg_cmtwacc( 7 downto  0);
					when X"8305" => datao_i <= reg_cmtwacc(15 downto  8);
					when X"8306" => datao_i <= reg_cmtwacc(23 downto 16);
					when X"8307" => datao_i <= reg_cmtwacc(31 downto 24);
					when X"8308" => datao_i <= reg_cmtwst( 7 downto  0);
					when X"8309" => datao_i <= reg_cmtwst(15 downto  8);
					when X"830A" => datao_i <= reg_cmtwst(23 downto 16);
					when X"830B" => datao_i <= reg_cmtwst(31 downto 24);
					when X"830C" => datao_i <= reg_acccnt( 7 downto  0);
					when X"830D" => datao_i <= reg_acccnt(15 downto  8);
					when X"830E" => datao_i <= reg_acccnt(23 downto 16);
					when X"830F" => datao_i <= reg_acccnt(31 downto 24);
					when others      =>
						if (add_f2(9) = '0') then
							datao_i <= buf_q;
						else
							datao_i <= X"FF";
						end if;
				end case;
			end if;
		end if;
	end process;


-- SD read data
	sdrdenb <= '1' when (indtenb_i = '1' and indtlt2_i = '1') else '0';

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			sd_rdcnt <= (others => '1');
			reg_sdrdfull <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (reg_sdrdinit = '1') then
				sd_rdcnt <= (others => '0');
				reg_sdrdfull <= '0';
			elsif (sdrdenb = '1') then
				if (sd_rdcnt = "111111111") then
					reg_sdrdfull <= '1';
				else
					sd_rdcnt <= sd_rdcnt + 1;
				end if;
			end if;
		end if;
	end process;

-- SD write data
	sdwrenb <= '1' when (outdtenb_i = '1' and indtlt2_i = '1') else '0';

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			sd_wrcnt <= (others => '1');
			reg_sdwrfull <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (reg_sdwrinit = '1') then
				sd_wrcnt <= (others => '0');
				reg_sdwrfull <= '0';
			elsif (sdwrenb = '1') then
				if (sd_wrcnt = "111111111") then
					reg_sdwrfull <= '1';
				else
					sd_wrcnt <= sd_wrcnt + 1;
				end if;
			end if;
		end if;
	end process;


-- CMT read request latch
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			cmtrdrq_f1 <= '0';
			cmtrdrq_f2 <= '0';
			cmtrdrq_f3 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			cmtrdrq_f1 <= CMTRDREQ;
			cmtrdrq_f2 <= cmtrdrq_f1;
			cmtrdrq_f3 <= cmtrdrq_f2;
		end if;
	end process;

-- fifo address for CMT read
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			fifo_sdramtmg <= (others => '0');
			fifo_cmtrdadd <= (others => '0');
			reg_cmtrdend  <= '0';
			reg_cmtrdfull <= '0';
			cmtrdycnt     <= (others => '0');
			cmtrdrdy_i    <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (reg_cmtrdinit = '1') then
				fifo_sdramtmg <= (others => '0');
				fifo_cmtrdadd <= (others => '0');
				reg_cmtrdend  <= '0';
				reg_cmtrdfull <= '0';
				cmtrdycnt     <= (others => '0');
				cmtrdrdy_i    <= '0';
			elsif (reg_cmtrdld = '1') then
				fifo_cmtrdadd <= reg_cmtptr_0;
			elsif (reg_cmtrdst = '1') then
				if (fifo_cmtrdadd = reg_cmtlen_0) then
					reg_cmtrdend  <= '1';
					reg_cmtrdfull <= '0';
				elsif (reg_cmtrdend = '0' and reg_cmtrdfull = '0') then

					if (reg_bufsel = "0101") then

						if (fifo_sdramtmg = "11111") then
							fifo_sdramtmg <= (others => '0');
							fifo_cmtrdadd <= fifo_cmtrdadd + 1;
							if (fifo_cmtrdadd(8 downto 0) = "111111111") then
								reg_cmtrdfull <= '1';
							end if;
						else
							fifo_sdramtmg <= fifo_sdramtmg + 1;
						end if;

					elsif (reg_bufsel = "0100") then

						if (cmtrdrq_f2 = '1' and cmtrdrq_f3 = '0') then
							cmtrdycnt     <= (others => '0');
							cmtrdrdy_i    <= '0';
							fifo_cmtrdadd <= fifo_cmtrdadd + 1;
							if (fifo_cmtrdadd(8 downto 0) = "111111111") then
								reg_cmtrdfull <= '1';
							end if;
						elsif (CMTRDACC = '0' and cmtrdycnt >= reg_cmtwst(23 downto 0) ) then
							cmtrdrdy_i    <= '1';
						elsif (CMTRDACC = '1' and CMTRDACCMD = '0' and cmtrdycnt >= reg_cmtwnom(23 downto 0) ) then
							cmtrdrdy_i    <= '1';
						elsif (CMTRDACC = '1' and CMTRDACCMD = '1' and cmtrdycnt >= reg_cmtwacc(23 downto 0) ) then
							cmtrdrdy_i    <= '1';
						else
							if (cmtrdopen_f2 = '0') then
								cmtrdycnt     <= (others => '0');
							elsif (detblk_f2 = '0') then
								cmtrdycnt     <= cmtrdycnt + 1;
							end if;
							cmtrdrdy_i    <= '0';
						end if;

					end if;

				end if;
			else
				reg_cmtrdfull <= '0';
			end if;
		end if;
	end process;


-- SDRAM write data generate
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			reg_rominit_f1 <= '0';
			reg_rominit_f2 <= '0';
			reg_rominit_f3 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			reg_rominit_f1 <= reg_rominit;
			reg_rominit_f2 <= reg_rominit_f1;
			reg_rominit_f3 <= reg_rominit_f2;
		end if;
	end process;

	reg_rominit_r <= '1' when (reg_rominit_f2 = '1' and reg_rominit_f3 = '0') else '0';

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			rominitcnt <= (others => '0');
			reg_rominitend <= '0';
		elsif (CLK50M'event and CLK50M = '1') then

			if (reg_rominit_r = '1') then
				reg_rominitend <= '1';
			elsif (rominitcnt = "11" & X"FFFF") then
				reg_rominitend <= '0';
			end if;

			if (reg_rominitend = '1') then
				rominitcnt <= rominitcnt + 1;
			else
				rominitcnt <= (others => '0');
			end if;

		end if;
	end process;


	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			sd_rdad_i  <= (others => '0');
			sd_rddt_i  <= (others => '0');
			sd_sdenb_i <= '0';
			sd_sdwrn_i <= '1';
		elsif (CLK50M'event and CLK50M = '1') then
			if (reg_rominitend = '1') then

				sd_rdad_i  <= "010" & "11" & rominitcnt(17 downto 4);
				sd_rddt_i  <= X"FF";
				sd_sdenb_i <= '1';

				if (rominitcnt(3 downto 2) = "10") then
					sd_sdwrn_i <= '0';
				else
					sd_sdwrn_i <= '1';
				end if;

			else

				sd_rdad_i <= ("000" & fifo_cmtrdadd(15 downto 0)) + reg_cmtptr_0(18 downto 0);
				sd_rddt_i <= buf_q;

				if (reg_cmtrdst = '1' and reg_bufsel = "0101") then
					sd_sdenb_i <= '1';
					if (fifo_sdramtmg(4 downto 3) = "10") then
						sd_sdwrn_i <= '0';
					else
						sd_sdwrn_i <= '1';
					end if;
				else
					sd_sdenb_i <= '0';
					sd_sdwrn_i <= '1';
				end if;

			end if;
		end if;
	end process;


-- CMT read data
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			cmtrddt_i <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			cmtrddt_i <= buf_q;
		end if;
	end process;


-- from 8049 data latch
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			cmtrdopen_f1 <= '0';
			cmtrdopen_f2 <= '0';
			cmtrdreq_f1  <= '0';
			cmtrdreq_f2  <= '0';
			cmtwropen_f1 <= '0';
			cmtwropen_f2 <= '0';
			cmtwrreq_f1  <= '0';
			cmtwrreq_f2  <= '0';
			cmtwrreq_f3  <= '0';
			cmtwrdt_f1   <= (others => '0');
			cmtwrdt_f2   <= (others => '0');
			cmtwrdt_f3   <= (others => '0');
			detblk_f1    <= '0';
			detblk_f2    <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			cmtrdopen_f1 <= CMTRDOPEN;
			cmtrdopen_f2 <= cmtrdopen_f1;
			cmtrdreq_f1  <= CMTRDREQ;
			cmtrdreq_f2  <= cmtrdreq_f1;
			cmtwropen_f1 <= CMTWROPEN;
			cmtwropen_f2 <= cmtwropen_f1;
			cmtwrreq_f1  <= CMTWRREQ;
			cmtwrreq_f2  <= cmtwrreq_f1;
			cmtwrreq_f3  <= cmtwrreq_f2;
			cmtwrdt_f1   <= CMTWRDT;
			cmtwrdt_f2   <= cmtwrdt_f1;
			cmtwrdt_f3   <= cmtwrdt_f2;
			detblk_f1    <= DETBLK;
			detblk_f2    <= detblk_f1;
		end if;
	end process;

-- CMT write data receive permission
	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			reg_cmtwrst_f1 <= '0';
			reg_cmtwrst_f2 <= '0';
			reg_cmtwrst_f3 <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			reg_cmtwrst_f1 <= reg_cmtwrst;
			reg_cmtwrst_f2 <= reg_cmtwrst_f1;
			reg_cmtwrst_f3 <= reg_cmtwrst_f2;
		end if;
	end process;

	cmtwrreq_r <= '1' when (cmtwrreq_f2 = '1' and cmtwrreq_f3 = '0') else '0';

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			cmtwrrdy_i    <= '0';
			reg_cmtwrfull <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (reg_cmtwrok = '0') then
				cmtwrrdy_i    <= '1';
				reg_cmtwrfull <= '0';
			elsif (cmtwropen_f2 = '0') then
				cmtwrrdy_i    <= '0';
				reg_cmtwrfull <= '0';
			elsif (cmtwrreq_r = '1') then
				if (fifo_cmtwradd(8 downto 0) = "111111111") then
					cmtwrrdy_i    <= '0';
					reg_cmtwrfull <= '1';
				else
					cmtwrrdy_i    <= '1';
					reg_cmtwrfull <= '0';
				end if;
			elsif (reg_cmtwrst_f2 = '1' and reg_cmtwrst_f3 = '0') then
				cmtwrrdy_i    <= '1';
				reg_cmtwrfull <= '0';
			end if;
		end if;
	end process;

	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			fifo_cmtwradd <= (others => '0');
		elsif (CLK50M'event and CLK50M = '1') then
			if (reg_cmtwrok = '0') then
				fifo_cmtwradd <= (others => '0');
			elsif (reg_cmtwrld = '1') then
				fifo_cmtwradd <= reg_cmtlen_1;
			elsif (cmtwrreq_r = '1') then
				fifo_cmtwradd <= fifo_cmtwradd + 1;
			end if;
		end if;
	end process;


	process (CLK50M,RSTN)
	begin
		if (RSTN = '0') then
			cmtwr_lt <= '0';
		elsif (CLK50M'event and CLK50M = '1') then
			if (reg_inuse = '1') then
				if (cmtwropen_f2 = '1') then
					cmtwr_lt <= '1';
				end if;
			else
				cmtwr_lt <= '0';
			end if;
		end if;
	end process;


	DATAO     <= datao_i;

	SD_RDAD   <= sd_rdad_i;
	SD_RDDT   <= sd_rddt_i;
	SD_SDENB  <= sd_sdenb_i;
	SD_SDWRN  <= sd_sdwrn_i;

	SD_RDDONE <= reg_sdrddone;
	CMTRDRDY  <= cmtrdrdy_i;
	CMTWRRDY  <= cmtwrrdy_i;
	CMTRDDT   <= cmtrddt_i;

	SD_ERR    <= reg_sderr;
	SD_OUTENB <= reg_inuse;

	ACCCNT    <= reg_acccnt;

	CMTCNT <=	fifo_cmtwradd(15 downto 0) when (cmtwropen_f2 = '1' or cmtwr_lt = '1') else
--				fifo_fddwradd(15 downto 0) when (reg_bufsel = "1110") else
--				fifo_fddrdadd(15 downto 0) when (reg_bufsel = "0110") else
				fifo_cmtrdadd(15 downto 0);

end RTL;
